@@ -1,14 +1,15 @@
 /*
  * tkMacOSXXStubs.c --
  *
- *	This file contains most of the X calls called by Tk. Many of
- *	these calls are just stubs and either don't make sense on the
- *	Macintosh or thier implamentation just doesn't do anything. Other
- *	calls will eventually be moved into other files.
+ *	This file contains most of the X calls called by Tk. Many of these
+ *	calls are just stubs and either don't make sense on the Macintosh or
+ *	their implamentation just doesn't do anything. Other calls will
+ *	eventually be moved into other files.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
- * Copyright 2001, Apple Computer, Inc.
- * Copyright (c) 2005-2007 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright 2001-2009, Apple Inc.
+ * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright 2014 Marc Culler.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -17,14 +18,13 @@
 #include "tkMacOSXPrivate.h"
 #include "tkMacOSXEvent.h"
 
-#if 0
 #include <IOKit/IOKitLib.h>
-#endif
+#include <IOKit/hidsystem/IOHIDShared.h>
 
 /*
  * Because this file is still under major development Debugger statements are
- * used through out this file. The define TCL_DEBUG will decide whether
- * the debugger statements actually call the debugger or not.
+ * used through out this file. The define TCL_DEBUG will decide whether the
+ * debugger statements actually call the debugger or not.
  */
 
 #ifndef TCL_DEBUG
@@ -33,33 +33,34 @@
 
 #define ROOT_ID 10
 
+CGFloat tkMacOSXZeroScreenHeight = 0;
+CGFloat tkMacOSXZeroScreenTop = 0;
+
 /*
  * Declarations of static variables used in this file.
  */
 
-static TkDisplay *gMacDisplay = NULL; /* Macintosh display. */
-static const char *macScreenName = ":0"; /* Default name of macintosh display. */
+static TkDisplay *gMacDisplay = NULL;
+				/* Macintosh display. */
+static const char *macScreenName = ":0";
+				/* Default name of macintosh display. */
 
 /*
  * Forward declarations of procedures used in this file.
  */
 
-static XID MacXIdAlloc(Display *display);
-static int DefaultErrorHandler(Display* display, XErrorEvent* err_evt);
+static XID		MacXIdAlloc(Display *display);
+static int		DefaultErrorHandler(Display *display,
+			    XErrorEvent *err_evt);
 
 /*
  * Other declarations
  */
 
-static int DestroyImage(XImage *image);
-static unsigned long ImageGetPixel(XImage *image, int x, int y);
-static int PutPixel(XImage *image, int x, int y, unsigned long pixel);
-#if 0
-static XImage *SubImage(XImage *image, int x, int y,
-	unsigned int width, unsigned int height);
-static int AddPixel(XImage *image, long value);
-#endif
-
+static int		DestroyImage(XImage *image);
+static unsigned long	ImageGetPixel(XImage *image, int x, int y);
+static int		ImagePutPixel(XImage *image, int x, int y,
+			    unsigned long pixel);
 
 /*
  *----------------------------------------------------------------------
@@ -79,40 +80,38 @@ static int AddPixel(XImage *image, long value);
  */
 
 void
-TkMacOSXDisplayChanged(Display *display)
+TkMacOSXDisplayChanged(
+    Display *display)
 {
-    GDHandle graphicsDevice;
     Screen *screen;
-    Rect bounds = {0, 0, 0, 0}, *maxBounds;
+    NSArray *nsScreens;
+
 
     if (display == NULL || display->screens == NULL) {
 	return;
     }
     screen = display->screens;
 
-    graphicsDevice = GetMainDevice();
-    screen->root_depth	= (*(*graphicsDevice)->gdPMap)->cmpSize *
-			       (*(*graphicsDevice)->gdPMap)->cmpCount;
-    screen->height	= (*graphicsDevice)->gdRect.bottom -
-	(*graphicsDevice)->gdRect.top;
-    screen->width	= (*graphicsDevice)->gdRect.right -
-	(*graphicsDevice)->gdRect.left;
+    nsScreens = [NSScreen screens];
+    if (nsScreens && [nsScreens count]) {
+	NSScreen *s = [nsScreens objectAtIndex:0];
+	NSRect bounds = [s frame], visible = [s visibleFrame];
+	NSRect maxBounds = NSZeroRect;
 
-    screen->mwidth	= (screen->width * 254 + 360) / 720;
-    screen->mheight	= (screen->height * 254 + 360) / 720;
+	tkMacOSXZeroScreenHeight = bounds.size.height;
+	tkMacOSXZeroScreenTop = tkMacOSXZeroScreenHeight -
+		(visible.origin.y + visible.size.height);
 
-    maxBounds = (Rect*) screen->ext_data;
-    *maxBounds = bounds;
-    graphicsDevice = GetDeviceList();
-    while (graphicsDevice) {
-	OSStatus err;
+	screen->root_depth = NSBitsPerPixelFromDepth([s depth]);
+	screen->width = bounds.size.width;
+	screen->height = bounds.size.height;
+	screen->mwidth = (bounds.size.width * 254 + 360) / 720;
+	screen->mheight = (bounds.size.height * 254 + 360) / 720;
 
-	err = ChkErr(GetAvailableWindowPositioningBounds, graphicsDevice,
-		&bounds);
-	if (err == noErr) {
-	    UnionRect(&bounds, maxBounds, maxBounds);
+	for (s in nsScreens) {
+	    maxBounds = NSUnionRect(maxBounds, [s visibleFrame]);
 	}
-	graphicsDevice = GetNextDevice(graphicsDevice);
+	*((NSRect *)screen->ext_data) = maxBounds;
     }
 }
 
@@ -121,8 +120,8 @@ TkMacOSXDisplayChanged(Display *display)
  *
  * TkpOpenDisplay --
  *
- *	Create the Display structure and fill it with device
- *	specific information.
+ *	Create the Display structure and fill it with device specific
+ *	information.
  *
  * Results:
  *	Returns a Display structure on success or NULL on failure.
@@ -135,12 +134,15 @@ TkMacOSXDisplayChanged(Display *display)
 
 TkDisplay *
 TkpOpenDisplay(
-    CONST char *display_name)
+    const char *display_name)
 {
     Display *display;
     Screen *screen;
     int fd = 0;
-    static Rect maxBounds = {0, 0, 0, 0};
+    static NSRect maxBounds = {{0, 0}, {0, 0}};
+    static char vendor[25] = "";
+    NSArray *cgVers;
+    NSAutoreleasePool *pool;
 
     if (gMacDisplay != NULL) {
 	if (strcmp(gMacDisplay->display->display_name, display_name) == 0) {
@@ -162,13 +164,25 @@ TkpOpenDisplay(
     display->screens	    = screen;
     display->nscreens	    = 1;
     display->default_screen = 0;
-    display->display_name   = (char*)macScreenName;
+    display->display_name   = (char *) macScreenName;
 
-    Gestalt(gestaltQuickdrawVersion, (long*)&display->proto_minor_version);
-    display->proto_major_version = 10;
-    display->proto_minor_version -= gestaltMacOSXQD;
-    display->vendor = "Apple";
-    Gestalt(gestaltSystemVersion, (long*)&display->release);
+    pool = [NSAutoreleasePool new];
+    cgVers = [[[NSBundle bundleWithIdentifier:@"com.apple.CoreGraphics"]
+	    objectForInfoDictionaryKey:@"CFBundleShortVersionString"]
+	    componentsSeparatedByString:@"."];
+    if ([cgVers count] >= 2) {
+	display->proto_major_version = [[cgVers objectAtIndex:1] integerValue];
+    }
+    if ([cgVers count] >= 3) {
+	display->proto_minor_version = [[cgVers objectAtIndex:2] integerValue];
+    }
+    if (!vendor[0]) {
+	snprintf(vendor, sizeof(vendor), "Apple AppKit %s %g",
+		([NSGarbageCollector defaultCollector] ? "GC" : "RR"),
+		NSAppKitVersionNumber);
+    }
+    display->vendor = vendor;
+    Gestalt(gestaltSystemVersion, (SInt32 *) &display->release);
 
     /*
      * These screen bits never change
@@ -177,7 +191,7 @@ TkpOpenDisplay(
     screen->display	= display;
     screen->black_pixel = 0x00000000 | PIXEL_MAGIC << 24;
     screen->white_pixel = 0x00FFFFFF | PIXEL_MAGIC << 24;
-    screen->ext_data	= (XExtData*) &maxBounds;
+    screen->ext_data	= (XExtData *) &maxBounds;
 
     screen->root_visual = (Visual *) ckalloc(sizeof(Visual));
     screen->root_visual->visualid     = 0;
@@ -191,17 +205,19 @@ TkpOpenDisplay(
     /*
      * Initialize screen bits that may change
      */
+
     TkMacOSXDisplayChanged(display);
 
     gMacDisplay = (TkDisplay *) ckalloc(sizeof(TkDisplay));
 
     /*
-     * This is the quickest way to make sure that all the *Init
-     * flags get properly initialized
+     * This is the quickest way to make sure that all the *Init flags get
+     * properly initialized
      */
 
     bzero(gMacDisplay, sizeof(TkDisplay));
     gMacDisplay->display = display;
+    [pool drain];
     return gMacDisplay;
 }
 
@@ -226,13 +242,14 @@ TkpCloseDisplay(
     TkDisplay *displayPtr)
 {
     Display *display = displayPtr->display;
+
     if (gMacDisplay != displayPtr) {
 	Tcl_Panic("TkpCloseDisplay: tried to call TkpCloseDisplay on bad display");
     }
 
     gMacDisplay = NULL;
-    if (display->screens != (Screen *) NULL) {
-	if (display->screens->root_visual != (Visual *) NULL) {
+    if (display->screens != NULL) {
+	if (display->screens->root_visual != NULL) {
 	    ckfree((char *) display->screens->root_visual);
 	}
 	ckfree((char *) display->screens);
@@ -245,11 +262,10 @@ TkpCloseDisplay(
  *
  * TkClipCleanup --
  *
- *	This procedure is called to cleanup resources associated with
- *	claiming clipboard ownership and for receiving selection get
- *	results. This function is called in tkWindow.c. This has to be
- *	called by the display cleanup function because we still need the
- *	access display elements.
+ *	This procedure is called to cleanup resources associated with claiming
+ *	clipboard ownership and for receiving selection get results. This
+ *	function is called in tkWindow.c. This has to be called by the display
+ *	cleanup function because we still need the access display elements.
  *
  * Results:
  *	None.
@@ -261,15 +277,15 @@ TkpCloseDisplay(
  */
 
 void
-TkClipCleanup(dispPtr)
-    TkDisplay *dispPtr; /* display associated with clipboard */
+TkClipCleanup(
+    TkDisplay *dispPtr)		/* display associated with clipboard */
 {
     /*
-     * Make sure that the local scrap is transfered to the global
-     * scrap if needed.
+     * Make sure that the local scrap is transfered to the global scrap if
+     * needed.
      */
 
-    TkSuspendClipboard();
+    [NSApp tkProvidePasteboard:dispPtr];
 
     if (dispPtr->clipWindow != NULL) {
 	Tk_DeleteSelHandler(dispPtr->clipWindow, dispPtr->clipboardAtom,
@@ -278,7 +294,7 @@ TkClipCleanup(dispPtr)
 		dispPtr->windowAtom);
 
 	Tk_DestroyWindow(dispPtr->clipWindow);
-	Tcl_Release((ClientData) dispPtr->clipWindow);
+	Tcl_Release(dispPtr->clipWindow);
 	dispPtr->clipWindow = NULL;
     }
 }
@@ -288,31 +304,31 @@ TkClipCleanup(dispPtr)
  *
  * MacXIdAlloc --
  *
- *	This procedure is invoked by Xlib as the resource allocator
- *	for a display.
+ *	This procedure is invoked by Xlib as the resource allocator for a
+ *	display.
  *
  * Results:
- *	The return value is an X resource identifier that isn't currently
- *	in use.
+ *	The return value is an X resource identifier that isn't currently in
+ *	use.
  *
  * Side effects:
- *	The identifier is removed from the stack of free identifiers,
- *	if it was previously on the stack.
+ *	The identifier is removed from the stack of free identifiers, if it
+ *	was previously on the stack.
  *
  *----------------------------------------------------------------------
  */
 
 static XID
 MacXIdAlloc(
-    Display *display)			/* Display for which to allocate. */
+    Display *display)		/* Display for which to allocate. */
 {
-	static long int cur_id = 100;
-	/*
-	 * Some special XIds are reserved
-	 *   - this is why we start at 100
-	 */
+    static long int cur_id = 100;
+    /*
+     * Some special XIds are reserved
+     *   - this is why we start at 100
+     */
 
-	return ++cur_id;
+    return ++cur_id;
 }
 
 /*
@@ -347,8 +363,8 @@ TkpWindowWasRecentlyDeleted(
  *
  * DefaultErrorHandler --
  *
- *	This procedure is the default X error handler. Tk uses it's
- *	own error handler so this call should never be called.
+ *	This procedure is the default X error handler. Tk uses it's own error
+ *	handler so this call should never be called.
  *
  * Results:
  *	None.
@@ -365,14 +381,14 @@ DefaultErrorHandler(
     XErrorEvent* err_evt)
 {
     /*
-     * This call should never be called. Tk replaces
-     * it with its own error handler.
+     * This call should never be called. Tk replaces it with its own error
+     * handler.
      */
+
     Tcl_Panic("Warning hit bogus error handler!");
     return 0;
 }
 
-
 char *
 XGetAtomName(
     Display * display,
@@ -383,7 +399,8 @@ XGetAtomName(
 }
 
 int
-_XInitImageFuncPtrs(XImage *image)
+_XInitImageFuncPtrs(
+    XImage *image)
 {
     return 0;
 }
@@ -396,24 +413,25 @@ XSetErrorHandler(
 }
 
 Window
-XRootWindow(Display *display, int screen_number)
+XRootWindow(
+    Display *display,
+    int screen_number)
 {
     display->request++;
     return ROOT_ID;
 }
 
 int
-XGetGeometry(display, d, root_return, x_return, y_return, width_return,
-	height_return, border_width_return, depth_return)
-    Display* display;
-    Drawable d;
-    Window* root_return;
-    int* x_return;
-    int* y_return;
-    unsigned int* width_return;
-    unsigned int* height_return;
-    unsigned int* border_width_return;
-    unsigned int* depth_return;
+XGetGeometry(
+    Display *display,
+    Drawable d,
+    Window *root_return,
+    int *x_return,
+    int *y_return,
+    unsigned int *width_return,
+    unsigned int *height_return,
+    unsigned int *border_width_return,
+    unsigned int *depth_return)
 {
     TkWindow *winPtr = ((MacDrawable *) d)->winPtr;
 
@@ -427,14 +445,11 @@ XGetGeometry(display, d, root_return, x_return, y_return, width_return,
 	*border_width_return = winPtr->changes.border_width;
 	*depth_return = Tk_Depth(winPtr);
     } else {
-	Rect boundsRect;
-	CGrafPtr destPort = TkMacOSXGetDrawablePort(d);
-
-	GetPortBounds(destPort, &boundsRect);
-	*x_return = boundsRect.left;
-	*y_return =  boundsRect.top;
-	*width_return = boundsRect.right - boundsRect.left;
-	*height_return = boundsRect.bottom - boundsRect.top;
+	CGSize size = ((MacDrawable *) d)->size;
+	*x_return = 0;
+	*y_return =  0;
+	*width_return = size.width;
+	*height_return = size.height;
 	*border_width_return = 0;
 	*depth_return = 32;
     }
@@ -469,7 +484,7 @@ XBell(
     Display* display,
     int percent)
 {
-    SysBeep(percent);
+    NSBeep();
     return Success;
 }
 
@@ -489,8 +504,8 @@ XSizeHints *
 XAllocSizeHints(void)
 {
     /*
-     * Always return NULL. Tk code checks to see if NULL
-     * is returned & does nothing if it is.
+     * Always return NULL. Tk code checks to see if NULL is returned & does
+     * nothing if it is.
      */
 
     return NULL;
@@ -648,7 +663,8 @@ XGetWindowProperty(
 }
 
 void
-XRefreshKeyboardMapping( XMappingEvent* x)
+XRefreshKeyboardMapping(
+    XMappingEvent *x)
 {
     /* used by tkXEvent.c */
     Debugger();
@@ -672,10 +688,11 @@ XForceScreenSaver(
     int mode)
 {
     /*
-     * This function is just a no-op. It is defined to
-     * reset the screen saver. However, there is no real
-     * way to do this on a Mac. Let me know if there is!
+     * This function is just a no-op. It is defined to reset the screen saver.
+     * However, there is no real way to do this on a Mac. Let me know if there
+     * is!
      */
+
     display->request++;
 }
 
@@ -712,7 +729,7 @@ XSetClipRectangles(
 
     while (n--) {
 	XRectangle rect = *rectangles;
-	
+
 	rect.x += clip_x_origin;
 	rect.y += clip_y_origin;
 	TkUnionRectWithRegion(&rect, clipRgn, clipRgn);
@@ -729,9 +746,9 @@ XSetClipRectangles(
  *
  * TkGetServerInfo --
  *
- *	Given a window, this procedure returns information about
- *	the window server for that window. This procedure provides
- *	the guts of the "winfo server" command.
+ *	Given a window, this procedure returns information about the window
+ *	server for that window. This procedure provides the guts of the "winfo
+ *	server" command.
  *
  * Results:
  *	None.
@@ -744,22 +761,25 @@ XSetClipRectangles(
 
 void
 TkGetServerInfo(
-    Tcl_Interp *interp,		/* The server information is returned in
-				 * this interpreter's result. */
-    Tk_Window tkwin)		/* Token for window; this selects a
-				 * particular display and server. */
+    Tcl_Interp *interp,		/* The server information is returned in this
+				 * interpreter's result. */
+    Tk_Window tkwin)		/* Token for window; this selects a particular
+				 * display and server. */
 {
-    char buffer[8 + TCL_INTEGER_SPACE * 2];
-    char buffer2[TCL_INTEGER_SPACE];
+    char buffer[5 + TCL_INTEGER_SPACE * 2];
+    char buffer2[11 + TCL_INTEGER_SPACE];
 
-    sprintf(buffer, "QD%dR%x ", ProtocolVersion(Tk_Display(tkwin)),
+    snprintf(buffer, sizeof(buffer), "CG%d.%d ",
+	    ProtocolVersion(Tk_Display(tkwin)),
 	    ProtocolRevision(Tk_Display(tkwin)));
-    sprintf(buffer2, " %x", VendorRelease(Tk_Display(tkwin)));
+    snprintf(buffer2, sizeof(buffer2), " Mac OS X %x",
+	    VendorRelease(Tk_Display(tkwin)));
     Tcl_AppendResult(interp, buffer, ServerVendor(Tk_Display(tkwin)),
 	    buffer2, NULL);
 }
 
 #pragma mark XImage handling
+
 /*
  *----------------------------------------------------------------------
  *
@@ -790,7 +810,6 @@ XCreateImage(
     int bytes_per_line)
 {
     XImage *ximage;
-
     display->request++;
     ximage = (XImage *) ckalloc(sizeof(XImage));
 
@@ -800,6 +819,7 @@ XCreateImage(
     ximage->xoffset = offset;
     ximage->format = format;
     ximage->data = data;
+    ximage->obdata = NULL;
 
     if (format == ZPixmap) {
 	ximage->bits_per_pixel = 32;
@@ -831,11 +851,10 @@ XCreateImage(
     ximage->red_mask = 0x00FF0000;
     ximage->green_mask = 0x0000FF00;
     ximage->blue_mask = 0x000000FF;
-    ximage->obdata = NULL;
     ximage->f.create_image = NULL;
     ximage->f.destroy_image = DestroyImage;
     ximage->f.get_pixel = ImageGetPixel;
-    ximage->f.put_pixel = PutPixel;
+    ximage->f.put_pixel = ImagePutPixel;
     ximage->f.sub_image = NULL;
     ximage->f.add_pixel = NULL;
 
@@ -850,8 +869,9 @@ XCreateImage(
  *	This function copies data from a pixmap or window into an XImage.
  *
  * Results:
- *	Returns a newly allocated image containing the data from the given
- *	rectangle of the given drawable.
+ *	Returns a newly allocated XImage containing the data from the given
+ *	rectangle of the given drawable, or NULL if the XImage could not be
+ *     constructed.
  *
  * Side effects:
  *	None.
@@ -870,44 +890,70 @@ XGetImage(
     unsigned long plane_mask,
     int format)
 {
-    XImage *   imagePtr = NULL;
-    Pixmap     pixmap = (Pixmap) NULL;
-    Tk_Window  win = (Tk_Window) ((MacDrawable *) d)->winPtr;
-    GC	       gc;
-    int	       depth = 32;
-    int	       offset = 0;
-    int	       bitmap_pad = 32;
-    int	       bytes_per_line = 0;
-
-    if (TkMacOSXGetDrawablePort(d)) {
-	if (format == ZPixmap) {
-	    if (width > 0 && height > 0) {
-		/* Tk_GetPixmap fails for zero width or height */
-		pixmap = Tk_GetPixmap(display, d, width, height, depth);
-	    }
-	    if (win) {
-		XGCValues values;
-		gc = Tk_GetGC(win, 0, &values);
-	    } else {
-		gc = XCreateGC(display, pixmap, 0, NULL);
-	    }
-	    if (pixmap) {
-		XCopyArea(display, d, pixmap, gc, x, y, width, height, 0, 0);
-	    }
-	    imagePtr = XCreateImage(display, NULL, depth, format, offset,
-		(char*)TkMacOSXGetDrawablePort(pixmap),
-		width, height, bitmap_pad, bytes_per_line);
-	    /* Track Pixmap underlying the XImage in the unused obdata field *
-	     * so that we can treat XImages coming from XGetImage specially. */
-	    imagePtr->obdata = (XPointer) pixmap;
-	    if (!win) {
-		XFreeGC(display, gc);
-	    }
-	} else {
-	    TkpDisplayWarning(
-		"XGetImage: only ZPixmap types are implemented",
-		"XGetImage Failure");
+    NSBitmapImageRep *bitmap_rep;
+    XImage *       imagePtr = NULL;
+    char *           bitmap = NULL;
+    char *           image_data=NULL;
+    int	        depth = 32;
+    int	        offset = 0;
+    int	        bitmap_pad = 0;
+    int	        bytes_per_row = 4*width;
+    int                size;
+    TkMacOSXDbgMsg("XGetImage");
+    if (format == ZPixmap) {
+	if (width == 0 || height == 0) {
+	    /* This happens all the time.
+	    TkMacOSXDbgMsg("XGetImage: empty image requested");
+	    */
+	    return NULL;
 	}
+
+	bitmap_rep =  BitmapRepFromDrawableRect(d, x, y,width, height);
+
+	if ( bitmap_rep == Nil                        ||
+	     [bitmap_rep bitmapFormat] != 0    || 
+	     [bitmap_rep samplesPerPixel] != 4 ||
+	     [bitmap_rep isPlanar] != 0               ) {
+	    TkMacOSXDbgMsg("XGetImage: Failed to construct NSBitmapRep");
+	    return NULL;
+	}
+
+	NSSize image_size = NSMakeSize(width, height);
+	NSImage* ns_image = [[NSImage alloc]initWithSize:image_size];
+	[ns_image addRepresentation:bitmap_rep];
+ 
+	/* Assume premultiplied nonplanar data with 4 bytes per pixel and alpha last.*/
+	if ( [bitmap_rep bitmapFormat] == 0 &&
+	     [bitmap_rep isPlanar ] == 0 &&
+	     [bitmap_rep samplesPerPixel] == 4 ) {
+	    bytes_per_row = [bitmap_rep bytesPerRow];
+	    size = bytes_per_row*height;
+	    image_data = (char*)[bitmap_rep bitmapData];
+	    if ( image_data ) {
+		int row, n, m;
+		bitmap = ckalloc(size);
+		/*
+		  Oddly enough, the bitmap has the top row at the beginning,
+		  and the pixels are in BGRA format.
+		*/
+		for (row=0, n=0; row<height; row++, n+=bytes_per_row) {
+		    for (m=n; m<n+bytes_per_row; m+=4) {
+			*(bitmap+m)     = *(image_data+m+2);
+			*(bitmap+m+1) = *(image_data+m+1);
+			*(bitmap+m+2) = *(image_data+m);
+			*(bitmap+m+3) = *(image_data+m+3);
+		    }
+		}
+	    }
+	}
+	if (bitmap) {
+	    imagePtr = XCreateImage(display, NULL, depth, format, offset,
+				    (char*)bitmap, width, height, bitmap_pad, bytes_per_row);
+	    [ns_image removeRepresentation:bitmap_rep]; /*releases the rep*/
+	    [ns_image release];
+	}
+    } else {
+	TkMacOSXDbgMsg("Could not extract image from drawable.");
     }
     return imagePtr;
 }
@@ -933,9 +979,7 @@ DestroyImage(
     XImage *image)
 {
     if (image) {
-	if (image->obdata) {
-	    Tk_FreePixmap((Display*) gMacDisplay, (Pixmap) image->obdata);
-	} else if (image->data) {
+	if (image->data) {
 	    ckfree(image->data);
 	}
 	ckfree((char*) image);
@@ -965,35 +1009,57 @@ ImageGetPixel(
     int x,
     int y)
 {
-    CGrafPtr destPort, savePort;
-    Boolean portChanged;
-    RGBColor cPix;
-    unsigned long r, g, b, c;
+    unsigned char r = 0, g = 0, b = 0;
 
-    destPort = (CGrafPtr)image->data;
-    portChanged = QDSwapPort(destPort, &savePort);
-    GetCPixel(x, y, &cPix);
-    if (image->obdata) {
-	/* Image from XGetImage, 16 bit color values */
-	r = (cPix . red) >> 8;
-	g = (cPix . green) >> 8;
-	b = (cPix . blue) >> 8;
-    } else {
-	r = cPix . red;
-	g = cPix . green;
-	b = cPix . blue;
+    if (image && image->data) {
+	unsigned char *srcPtr = ((unsigned char*) image->data)
+		+ (y * image->bytes_per_line)
+		+ (((image->xoffset + x) * image->bits_per_pixel) / NBBY);
+
+	switch (image->bits_per_pixel) {
+	    case 32: {
+		r = (*((unsigned int*) srcPtr) >> 16) & 0xff;
+		g = (*((unsigned int*) srcPtr) >>  8) & 0xff;
+		b = (*((unsigned int*) srcPtr)      ) & 0xff;
+		/*if (image->byte_order == LSBFirst) {
+		    r = srcPtr[2]; g = srcPtr[1]; b = srcPtr[0];
+		} else {
+		    r = srcPtr[1]; g = srcPtr[2]; b = srcPtr[3];
+		}*/
+		break;
+	    }
+	    case 16:
+		r = (*((unsigned short*) srcPtr) >> 7) & 0xf8;
+		g = (*((unsigned short*) srcPtr) >> 2) & 0xf8;
+		b = (*((unsigned short*) srcPtr) << 3) & 0xf8;
+		break;
+	    case 8:
+		r = (*srcPtr << 2) & 0xc0;
+		g = (*srcPtr << 4) & 0xc0;
+		b = (*srcPtr << 6) & 0xc0;
+		r |= r >> 2 | r >> 4 | r >> 6;
+		g |= g >> 2 | g >> 4 | g >> 6;
+		b |= b >> 2 | b >> 4 | b >> 6;
+		break;
+	    case 4: {
+		unsigned char c = (x % 2) ? *srcPtr : (*srcPtr >> 4);
+		r = (c & 0x04) ? 0xff : 0;
+		g = (c & 0x02) ? 0xff : 0;
+		b = (c & 0x01) ? 0xff : 0;
+		break;
+		}
+	    case 1:
+		r = g = b = ((*srcPtr) & (0x80 >> (x % 8))) ? 0xff : 0;
+		break;
+	}
     }
-    c = (r<<16)|(g<<8)|(b);
-    if (portChanged) {
-	QDSwapPort(savePort, NULL);
-    }
-    return c;
+    return (PIXEL_MAGIC << 24) | (r << 16) | (g << 8) | b;
 }
 
 /*
  *----------------------------------------------------------------------
  *
- * PutPixel --
+ * ImagePutPixel --
  *
  *	Set a single pixel in an image.
  *
@@ -1007,61 +1073,53 @@ ImageGetPixel(
  */
 
 static int
-PutPixel(
+ImagePutPixel(
     XImage *image,
     int x,
     int y,
     unsigned long pixel)
 {
-    CGrafPtr destPort, savePort;
-    Boolean portChanged;
-    RGBColor cPix;
-    unsigned long r, g, b;
+    if (image && image->data) {
+	unsigned char r = ((pixel & image->red_mask)   >> 16) & 0xff;
+	unsigned char g = ((pixel & image->green_mask) >>  8) & 0xff;
+	unsigned char b = ((pixel & image->blue_mask)       ) & 0xff;
+	unsigned char *dstPtr = ((unsigned char*) image->data)
+		+ (y * image->bytes_per_line)
+		+ (((image->xoffset + x) * image->bits_per_pixel) / NBBY);
 
-    destPort = (CGrafPtr)image->data;
-    portChanged = QDSwapPort(destPort, &savePort);
-    r  = (pixel & image->red_mask)>>16;
-    g  = (pixel & image->green_mask)>>8;
-    b  = (pixel & image->blue_mask);
-    if (image->obdata) {
-	/* Image from XGetImage, 16 bit color values */
-	cPix . red = r << 8;
-	cPix . green = g << 8;
-	cPix . blue = b << 8;
-    } else {
-	cPix . red = r;
-	cPix . green = g;
-	cPix . blue = b;
-    }
-    SetCPixel(x, y, &cPix);
-    if (portChanged) {
-	QDSwapPort(savePort, NULL);
+	switch (image->bits_per_pixel) {
+	    case 32:
+		*((unsigned int*) dstPtr) = (0xff << 24) | (r << 16) |
+			(g << 8) | b;
+		/*if (image->byte_order == LSBFirst) {
+		    dstPtr[3] = 0xff; dstPtr[2] = r; dstPtr[1] = g; dstPtr[0] = b;
+		} else {
+		    dstPtr[0] = 0xff; dstPtr[1] = r; dstPtr[2] = g; dstPtr[3] = b;
+		}*/
+		break;
+	    case 16:
+		*((unsigned short*) dstPtr) = ((r & 0xf8) << 7) |
+			((g & 0xf8) << 2) | ((b & 0xf8) >> 3);
+		break;
+	    case 8:
+		*dstPtr = ((r & 0xc0) >> 2) | ((g & 0xc0) >> 4) |
+			((b & 0xc0) >> 6);
+		break;
+	    case 4: {
+		unsigned char c = ((r & 0x80) >> 5) | ((g & 0x80) >> 6) |
+			((b & 0x80) >> 7);
+		*dstPtr = (x % 2) ? ((*dstPtr & 0xf0) | (c & 0x0f)) :
+			((*dstPtr & 0x0f) | ((c << 4) & 0xf0));
+		break;
+		}
+	    case 1:
+		*dstPtr = ((r|g|b) & 0x80) ? (*dstPtr | (0x80 >> (x % 8))) :
+			(*dstPtr & ~(0x80 >> (x % 8)));
+		break;
+	}
     }
     return 0;
 }
-
-#if 0
-static XImage *
-SubImage(
-    XImage *image,
-    int x,
-    int y,
-    unsigned int width,
-    unsigned int height)
-{
-    Debugger();
-    return NULL;
-}
-
-static int
-AddPixel(
-    XImage *image,
-    long value)
-{
-    Debugger();
-    return 0;
-}
-#endif
 
 /*
  *----------------------------------------------------------------------
@@ -1070,8 +1128,8 @@ AddPixel(
  * XSetWindowBackgroundPixmap, XSetWindowBorder, XSetWindowBorderPixmap,
  * XSetWindowBorderWidth, XSetWindowColormap
  *
- *	These functions are all no-ops. They all have equivilent
- *	Tk calls that should always be used instead.
+ *	These functions are all no-ops. They all have equivalent Tk calls that
+ *	should always be used instead.
  *
  * Results:
  *	None.
@@ -1084,24 +1142,24 @@ AddPixel(
 
 void
 XChangeWindowAttributes(
-    Display* display,
+    Display *display,
     Window w,
     unsigned long value_mask,
-    XSetWindowAttributes* attributes)
+    XSetWindowAttributes *attributes)
 {
 }
 
 void
 XSetWindowBackground(
-	Display *display,
-	Window window,
-	unsigned long value)
+    Display *display,
+    Window window,
+    unsigned long value)
 {
 }
 
 void
 XSetWindowBackgroundPixmap(
-    Display* display,
+    Display *display,
     Window w,
     Pixmap background_pixmap)
 {
@@ -1109,7 +1167,7 @@ XSetWindowBackgroundPixmap(
 
 void
 XSetWindowBorder(
-    Display* display,
+    Display *display,
     Window w,
     unsigned long border_pixel)
 {
@@ -1117,7 +1175,7 @@ XSetWindowBorder(
 
 void
 XSetWindowBorderPixmap(
-    Display* display,
+    Display *display,
     Window w,
     Pixmap border_pixmap)
 {
@@ -1125,7 +1183,7 @@ XSetWindowBorderPixmap(
 
 void
 XSetWindowBorderWidth(
-    Display* display,
+    Display *display,
     Window w,
     unsigned int width)
 {
@@ -1133,7 +1191,7 @@ XSetWindowBorderWidth(
 
 void
 XSetWindowColormap(
-    Display* display,
+    Display *display,
     Window w,
     Colormap colormap)
 {
@@ -1142,24 +1200,25 @@ XSetWindowColormap(
 
 Status
 XStringListToTextProperty(
-    char** list,
+    char **list,
     int count,
-    XTextProperty* text_prop_return)
+    XTextProperty *text_prop_return)
 {
     Debugger();
     return (Status) 0;
 }
+
 void
 XSetWMClientMachine(
-    Display* display,
+    Display *display,
     Window w,
-    XTextProperty* text_prop)
+    XTextProperty *text_prop)
 {
     Debugger();
 }
+
 XIC
-XCreateIC(
-    void)
+XCreateIC(void)
 {
     Debugger();
     return (XIC) 0;
@@ -1182,10 +1241,10 @@ XCreateIC(
  *----------------------------------------------------------------------
  */
 
-CONST char *
+const char *
 TkGetDefaultScreenName(
     Tcl_Interp *interp,		/* Not used. */
-    CONST char *screenName)		/* If NULL, use default string. */
+    const char *screenName)		/* If NULL, use default string. */
 {
 #if 0
     if ((screenName == NULL) || (screenName[0] == '\0')) {
@@ -1195,7 +1254,6 @@ TkGetDefaultScreenName(
 #endif
     return macScreenName;
 }
-#if 0
 
 /*
  *----------------------------------------------------------------------
@@ -1205,8 +1263,8 @@ TkGetDefaultScreenName(
  *	Return the number of milliseconds the user was inactive.
  *
  * Results:
- *	The number of milliseconds the user has been inactive,
- *	or -1 if querying the inactive time is not supported.
+ *	The number of milliseconds the user has been inactive, or -1 if
+ *	querying the inactive time is not supported.
  *
  * Side effects:
  *	None.
@@ -1214,13 +1272,15 @@ TkGetDefaultScreenName(
  */
 
 long
-Tk_GetUserInactiveTime(Display *dpy)
+Tk_GetUserInactiveTime(
+    Display *dpy)
 {
     io_registry_entry_t regEntry;
     CFMutableDictionaryRef props = NULL;
     CFTypeRef timeObj;
     long ret = -1l;
     uint64_t time;
+    IOReturn result;
 
     regEntry = IOServiceGetMatchingService(kIOMasterPortDefault,
 	    IOServiceMatching("IOHIDSystem"));
@@ -1229,7 +1289,7 @@ Tk_GetUserInactiveTime(Display *dpy)
 	return -1l;
     }
 
-    IOReturn result = IORegistryEntryCreateCFProperties(regEntry, &props,
+    result = IORegistryEntryCreateCFProperties(regEntry, &props,
 	    kCFAllocatorDefault, 0);
     IOObjectRelease(regEntry);
 
@@ -1247,13 +1307,13 @@ Tk_GetUserInactiveTime(Display *dpy)
 		    CFRangeMake(0, sizeof(time)), (UInt8 *) &time);
 	    /* Convert nanoseconds to milliseconds. */
 	    /* ret /= kMillisecondScale; */
-	    ret = (long)(time/kMillisecondScale);
+	    ret = (long) (time/kMillisecondScale);
 	} else if (type == CFNumberGetTypeID()) { /* Panther+ */
 	    CFNumberGetValue((CFNumberRef)timeObj,
 		    kCFNumberSInt64Type, &time);
 	    /* Convert nanoseconds to milliseconds. */
 	    /* ret /= kMillisecondScale; */
-	    ret = (long)(time/kMillisecondScale);
+	    ret = (long) (time/kMillisecondScale);
 	} else {
 	    ret = -1l;
 	}
@@ -1275,15 +1335,45 @@ Tk_GetUserInactiveTime(Display *dpy)
  *	none
  *
  * Side effects:
- *	The user inactivity timer of the underlaying windowing system
- *	is reset to zero.
+ *	The user inactivity timer of the underlaying windowing system is reset
+ *	to zero.
  *
  *----------------------------------------------------------------------
  */
 
 void
-Tk_ResetUserInactiveTime(Display *dpy)
+Tk_ResetUserInactiveTime(
+    Display *dpy)
 {
-    UpdateSystemActivity(UsrActivity);
+    IOGPoint loc;
+    kern_return_t kr;
+    NXEvent nullEvent = {NX_NULLEVENT, {0, 0}, 0, -1, 0};
+    enum { kNULLEventPostThrottle = 10 };
+    static io_connect_t io_connection = MACH_PORT_NULL;
+
+    if (io_connection == MACH_PORT_NULL) {
+	io_service_t service = IOServiceGetMatchingService(
+		kIOMasterPortDefault, IOServiceMatching(kIOHIDSystemClass));
+
+	if (service == MACH_PORT_NULL) {
+	    return;
+	}
+	kr = IOServiceOpen(service, mach_task_self(), kIOHIDParamConnectType,
+		&io_connection);
+	IOObjectRelease(service);
+	if (kr != KERN_SUCCESS) {
+	    return;
+	}
+    }
+    kr = IOHIDPostEvent(io_connection, NX_NULLEVENT, loc, &nullEvent.data,
+	    FALSE, 0, FALSE);
 }
-#endif
+
+/*
+ * Local Variables:
+ * mode: objc
+ * c-basic-offset: 4
+ * fill-column: 79
+ * coding: utf-8
+ * End:
+ */
